package boxes.AlphabetizerAbs.Alphabetizer;

import datatypes.systems.*;
import interfaces.IAlphabetize;
import interfaces.IStoreHelper;
import interfaces.ILine;
import datatypes.Word;
import datatypes.LStore;

// IStoreHelper is an internal interface in box Alphabetizer 
// and not exposed 
public class IapImp implements IAlphabetize, IStoreHelper
{
	private BoxTop _box;
	private ILine ial;
	private LStore store;
	private int size;
	
	public IapImp(BoxTop mybox)
	{
		_box = mybox;
	}

	public void alphabetize()
	{
		init();
		copyToLocalStore(); 
		sort();
	}
	
	public void sort()
	{
		quicksort(0, store.size()-1);
	}
	
	public LStore getStoreRef()
	{
		return store;	
	}
	
	//////////////////// private methods
	private void init()
	{
		// set for required interfaces
		// IAccessLines line;
		InterfaceName name = new InterfaceName("line");
		ial = (ILine)_box.getRequiredItf(name);
		size = ial.getTotalLineNo();
		store = new LStore();
	}
	
	private void copyToLocalStore()
	{
		for (int i = 0; i < size; i++)
			store.add(i, ial.getLine(i));	

	}
	
	private void quicksort(int lf, int rt)
	{
	       if (lf >= rt) return;
	       int p = part(lf, rt);
	       quicksort(lf, p-1);
	       quicksort(p + 1, rt);
	}
 
   	private int part(int lf, int rt)
    	{
	      Word [] pivot = store.get(lf);
	 
	      int left = lf;
	      int right = rt;
	      while (left < right)
	      {
	      	 if (lessThan(store.get(left),pivot)) 	// < 
	         	left++;
	         else if (greaterThan(store.get(right),pivot))
	         	right --;
	         else
	         	swap(left, right);
	      }
	      return right;
   	}

    	private void swap(int i, int j)
    	{
	       Word [] temp = store.get(i);
	       store.set(i, store.get(j)) ;
	       store.set(j,temp);
   	}
   	
   	// >=
   	private boolean greaterThan(Word[] w1, Word[] w2)
   	{
   		int size_1 = w1.length;
   		int size_2 = w2.length;
   		
   		int size = size_1;
   		if (size_2 < size)
   			size = size_2;
   		
   		for (int i = 0; i < size; i++)
   		{
   			if (w1[i].compareTo(w2[i]) > 0)  
   				return true;
   			if (w1[i].compareTo(w2[i]) < 0)	
   				return false;	
   			if (w1[i].equals(w2[i]))
   				i++;	
   		}
   		
   		if (size_1 >= size_2)
   			return true;
   		return false;
   	}
   	
   	// <
   	private boolean lessThan(Word[] w1, Word[] w2)  
   	{
   		return (!greaterThan(w1, w2));
   	}
}